/*!
 * Automatically forwards events to Snowplow mobile trackers running in a WebView. v4.6.3 (https://github.com/snowplow/snowplow-javascript-tracker)
 * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
 * Licensed under BSD-3-Clause
 */

import { hasMobileInterface, trackWebViewEvent } from '@snowplow/webview-tracker';

/**
 * Decodes base64-encoded strings.
 * Copied from the core library.
 */
function base64urldecode(data) {
    if (!data) {
        return data;
    }
    var padding = 4 - (data.length % 4);
    switch (padding) {
        case 2:
            data += '==';
            break;
        case 3:
            data += '=';
            break;
    }
    var b64Data = data.replace(/-/g, '+').replace(/_/g, '/');
    return base64decode(b64Data);
}
var b64 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
function base64decode(encodedData) {
    //  discuss at: http://locutus.io/php/base64_decode/
    // original by: Tyler Akins (http://rumkin.com)
    // improved by: Thunder.m
    // improved by: Kevin van Zonneveld (http://kvz.io)
    // improved by: Kevin van Zonneveld (http://kvz.io)
    //    input by: Aman Gupta
    //    input by: Brett Zamir (http://brett-zamir.me)
    // bugfixed by: Onno Marsman (https://twitter.com/onnomarsman)
    // bugfixed by: Pellentesque Malesuada
    // bugfixed by: Kevin van Zonneveld (http://kvz.io)
    // improved by: Indigo744
    //   example 1: base64_decode('S2V2aW4gdmFuIFpvbm5ldmVsZA==')
    //   returns 1: 'Kevin van Zonneveld'
    //   example 2: base64_decode('YQ==')
    //   returns 2: 'a'
    //   example 3: base64_decode('4pyTIMOgIGxhIG1vZGU=')
    //   returns 3: '✓ à la mode'
    // decodeUTF8string()
    // Internal function to decode properly UTF8 string
    // Adapted from Solution #1 at https://developer.mozilla.org/en-US/docs/Web/API/WindowBase64/Base64_encoding_and_decoding
    var decodeUTF8string = function (str) {
        // Going backwards: from bytestream, to percent-encoding, to original string.
        return decodeURIComponent(str
            .split('')
            .map(function (c) {
            return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
        })
            .join(''));
    };
    var o1, o2, o3, h1, h2, h3, h4, bits, i = 0, ac = 0, dec = '';
    var tmpArr = [];
    if (!encodedData) {
        return encodedData;
    }
    encodedData += '';
    do {
        // unpack four hexets into three octets using index points in b64
        h1 = b64.indexOf(encodedData.charAt(i++));
        h2 = b64.indexOf(encodedData.charAt(i++));
        h3 = b64.indexOf(encodedData.charAt(i++));
        h4 = b64.indexOf(encodedData.charAt(i++));
        bits = (h1 << 18) | (h2 << 12) | (h3 << 6) | h4;
        o1 = (bits >> 16) & 0xff;
        o2 = (bits >> 8) & 0xff;
        o3 = bits & 0xff;
        if (h3 === 64) {
            tmpArr[ac++] = String.fromCharCode(o1);
        }
        else if (h4 === 64) {
            tmpArr[ac++] = String.fromCharCode(o1, o2);
        }
        else {
            tmpArr[ac++] = String.fromCharCode(o1, o2, o3);
        }
    } while (i < encodedData.length);
    dec = tmpArr.join('');
    return decodeUTF8string(dec.replace(/\0+$/, ''));
}

/**
 * Forwards events to Snowplow mobile trackers running in a WebView.
 * @param configuration - Configuration. Specify certain tracker namespaces to forward events to.
 */
function WebViewPlugin(configuration) {
    var LOG;
    return {
        logger: function (logger) {
            LOG = logger;
        },
        filter: function (payload) {
            var _a;
            if (hasMobileInterface() === true) {
                LOG.debug("Payload (event ID: ".concat(payload.eid, ") was filtered out and forwarded to WebView tracker."));
                var atomicProperties = {
                    eventName: payload.e,
                    trackerVersion: payload.tv,
                    useragent: (_a = payload.ua) !== null && _a !== void 0 ? _a : window.navigator.userAgent,
                    pageUrl: payload.url,
                    pageTitle: payload.page,
                    referrer: payload.refr,
                    category: payload.se_ca,
                    action: payload.se_ac,
                    label: payload.se_la,
                    property: payload.se_pr,
                    value: payload.se_va !== undefined ? parseFloat(payload.se_va) : undefined,
                    pingXOffsetMin: payload.pp_mix !== undefined ? parseInt(payload.pp_mix) : undefined,
                    pingXOffsetMax: payload.pp_max !== undefined ? parseInt(payload.pp_max) : undefined,
                    pingYOffsetMin: payload.pp_miy !== undefined ? parseInt(payload.pp_miy) : undefined,
                    pingYOffsetMax: payload.pp_may !== undefined ? parseInt(payload.pp_may) : undefined,
                };
                var event_1 = getSelfDescribingEventData(payload);
                var entities = getEntities(payload);
                var trackers = configuration === null || configuration === void 0 ? void 0 : configuration.trackerNamespaces;
                trackWebViewEvent({ properties: atomicProperties, event: event_1, context: entities }, trackers);
                return false;
            }
            else {
                return true;
            }
        },
    };
}
function getSelfDescribingEventData(payload) {
    if (payload.ue_pr) {
        return JSON.parse(payload.ue_pr).data;
    }
    else if (payload.ue_px) {
        var decoded = base64urldecode(payload.ue_px);
        return JSON.parse(decoded).data;
    }
    return undefined;
}
function getEntities(payload) {
    if (payload.co) {
        return JSON.parse(payload.co)['data'];
    }
    else if (payload.cx) {
        var decoded = base64urldecode(payload.cx);
        return JSON.parse(decoded)['data'];
    }
    return [];
}

export { WebViewPlugin };
//# sourceMappingURL=index.module.js.map
